@@ -11,8 +11,8 @@
 #
 # See http://www.latex-project.org/lppl.txt for details.
 #
-# Running otfinfo on Linux
+# Running pdffonts on Linux
 #
 #CODEbegin
-otfinfo -h
+pdffonts 02-03-3.pdf
 #CODEend
