@@ -2,15 +2,16 @@
 -----------------------------------------------------------------------
 --         FILE:  luaotfload-tool.lua
 --  DESCRIPTION:  database functionality
--- REQUIREMENTS:  luaotfload 2.6
+-- REQUIREMENTS:  luaotfload 2.8
 --       AUTHOR:  Khaled Hosny, Élie Roux, Philipp Gesang
 --      LICENSE:  GPL v2.0
 -----------------------------------------------------------------------
 
-luaotfload          = luaotfload or { }
-local version       = "2.6"
-luaotfload.version  = version
-luaotfload.self     = "luaotfload-tool"
+luaotfload                     = luaotfload or { }
+local version                  = "2.8"
+luaotfload.version             = version
+luaotfload.min_luatex_version  = { 0, 95, 0 }
+luaotfload.self                = "luaotfload-tool"
 
 --[[doc--
 
@@ -33,16 +34,6 @@ see the luaotfload documentation for more info. Report bugs to
 
 kpse.set_program_name "luatex"
 
---[[doc--
-
-    We test for Lua 5.1 by means of capability detection to see if
-    we’re running an outdated Luatex.  If so, we bail.
-
-    \url{http://lua-users.org/wiki/LuaVersionCompatibility}
-
---doc]]--
-
-
 local iowrite         = io.write
 local kpsefind_file   = kpse.find_file
 local mathfloor       = math.floor
@@ -59,19 +50,33 @@ local texiowrite      = texio.write
 local tonumber        = tonumber
 local type            = type
 
-local runtime
-if _G.getfenv ~= nil then -- 5.1 or LJ
-    if _G.jit ~= nil then
-        runtime = { "jit", jit.version }
-    else
-        runtime = { "stock", _VERSION }
-        print "FATAL ERROR"
-        print "Luaotfload requires a Luatex version >=0.76."
-        print "Please update your TeX distribution!"
-        os.exit (-1)
+do
+    local runtime         = _G.jit and { "jit"  , jit.version }
+                                    or { "stock", _VERSION }
+    local stats           = status and status.list ()
+    local minimum         = luaotfload.min_luatex_version
+    local actual          = { 0, 0, 0 }
+    if stats then
+        local major    = stats.luatex_version / 100
+        local minor    = stats.luatex_version % 100
+        local revision = stats.luatex_revision --[[ : string ]]
+        local revno    = tonumber (revision)
+        actual         = { major, minor, revno or 0 }
     end
-else -- 5.2
-    runtime = { "stock", _VERSION }
+
+    if actual [1] < minimum [1]
+    or actual == minimum and actual [2] < minimum [2]
+    or actual == minimum and actual [2] == minimum [2] and actual [3] < minimum [3]
+    then
+        texio.write_nl ("term and log",
+                        string.format ("\tFATAL ERROR\n\z
+                                        \tLuaotfload requires a Luatex version >= %d.%d.%d.\n\z
+                                        \tPlease update your TeX distribution!\n\n",
+                                       (unpack or table.unpack) (minimum)))
+        error "version check failed"
+    end
+    luaotfload.runtime        = runtime
+    luaotfload.luatex_version = actual
 end
 
 local C, Ct, P, S  = lpeg.C, lpeg.Ct, lpeg.P, lpeg.S
@@ -104,6 +109,7 @@ local iosavedata                = io.savedata
 local lfsisdir                  = lfs.isdir
 local lfsisfile                 = lfs.isfile
 local stringsplit               = string.split
+local tablekeys                 = table.keys
 local tableserialize            = table.serialize
 local tablesortedkeys           = table.sortedkeys
 local tabletohash               = table.tohash
@@ -130,11 +136,29 @@ local backup = {
 
 texio.write, texio.write_nl          = dummy_function, dummy_function
 require "fontloader-basics-gen.lua"
-
 texio.write, texio.write_nl          = backup.write, backup.write_nl
 utilities                            = backup.utilities
 
-fonts = { names = { } } -- for db; normally provided by the fontloaders
+pdf = pdf or { } --- for font-tfm
+
+require "fontloader-data-con"
+require "fontloader-font-ini"
+require "fontloader-font-con"
+require "fontloader-fonts-enc"
+require "fontloader-font-cid"
+require "fontloader-font-map"
+require "fontloader-font-oti"
+require "fontloader-font-otr"
+require "fontloader-font-cff"
+require "fontloader-font-ttf"
+require "fontloader-font-dsp"
+require "fontloader-font-oup"
+require "fontloader-font-onr"
+require "fontloader-font-def"
+
+fonts = fonts or { }
+local fontsnames = fonts.names or { }
+fonts.names      = fontsnames
 
 local require_init = { }
 
@@ -157,7 +181,7 @@ end
 
 require "alt_getopt"
 
-loadmodule "log.lua"       --- this populates the luaotfload.log.* namespace
+loadmodule "log"           --- this populates the luaotfload.log.* namespace
 loadmodule "parsers"       --- fonts.conf, configuration, and request syntax
 loadmodule "configuration" --- configuration file handling
 loadmodule "database"
@@ -239,8 +263,6 @@ Usage: %s [OPTIONS...]
 
   -i --info                    display basic font metadata
   -I --inspect                 display detailed font metadata
-  -w --warnings                display warnings generated by the
-                               fontloader library
 
   --list=<criterion>           output list of entries by field <criterion>
   --list=<criterion>:<value>   restrict to entries with <criterion>=<value>
@@ -310,14 +332,21 @@ local version_msg = function ( )
     local out   = function (...) texiowrite_nl (stringformat (...)) end
     local uname = os.uname ()
     local meta  = fonts.names.getmetadata ()
+
+    local runtime = luaotfload.runtime
+    local actual  = luaotfload.luatex_version
+    local status  = config.luaotfload.status
+    local notes   = status and status.notes or { }
+
     out (about, luaotfload.self)
     out ("%s version: %q", luaotfload.self, version)
-    out ("Revision: %q", config.luaotfload.status.notes.revision)
+    if notes.description then
+        out ("Luaotfload: %q", notes.description)
+    end
+    out ("Revision: %q", notes.revision)
     out ("Lua interpreter: %s; version %q", runtime[1], runtime[2])
-    out ("Luatex SVN revision: %d", status.luatex_svn)
-    out ("Luatex version: %.2f.%d",
-         status.luatex_version / 100,
-         status.luatex_revision)
+--[[out ("Luatex SVN revision: %d", info.luatex_svn)]] --> SVN r5624
+    out ("Luatex version: %d.%d", actual [1], actual [2])
     out ("Platform: type=%s name=%s", os.type, os.name)
 
     local uname_vars = tablesortedkeys (uname)
@@ -329,7 +358,7 @@ local version_msg = function ( )
         out("No database metadata available.")
     else
         out ("Index: version=%q created=%q modified=%q",
-             config.luaotfload.status.notes.revision,
+             meta.version or "too old",
              meta.created or "ages ago",
              meta.modified or "ages ago")
     end
@@ -363,6 +392,7 @@ local format_counter = function (stack)
 end
 
 local print_heading = function (title, level)
+    if not title then return end
     local structuredata
     if currentdepth == level then -- top is current
         counterstack[#counterstack] = counterstack[#counterstack] + 1
@@ -397,7 +427,7 @@ local baseindent = "    "
 --[[doc--
 
     show_info_items -- Together with show_info_table prints the table returned by
-    fontloader.info(), recursing into nested tables if appropriate (as necessitated
+    readers.getinfo(), recursing into nested tables if appropriate (as necessitated
     by Luatex versions 0.78+ which include the pfminfo table in the result.
 
 --doc]]--
@@ -425,29 +455,6 @@ local show_info_items = function (fontinfo)
     texiowrite_nl ""
 end
 
-local p_eol     = S"\n\r"^1
-local p_space   = S" \t\v"^0
-local p_line    = p_space * C((1 - p_eol)^1)^-1
-local p_lines   = Ct(p_line * (p_eol^1 * p_line^-1)^0)
-
-local show_fontloader_warnings = function (ws)
-    local nws = #ws
-    print_heading(stringformat(
-        [[the fontloader emitted %d warnings]],
-        nws), 2)
-    texiowrite_nl ""
-    for i=1, nws do
-        local w = ws[i]
-        texiowrite_nl (stringformat("%d:", i))
-        local lines = lpegmatch(p_lines, w)
-        for i=1, #lines do
-            local line = lines[i]
-            texiowrite_nl("  · " .. line)
-        end
-        texiowrite_nl ""
-    end
-end
-
 local p_spacechar  = S" \n\r\t\v"
 local p_wordchar   = (1 - p_spacechar)
 local p_whitespace = p_spacechar^1
@@ -457,15 +464,16 @@ local p_words      = Ct(p_word * (p_whitespace * p_word)^0)
 --- string -> int -> string list
 local reflow = function (text, width)
     local words
-    if type(text) == "string" then
+    local t_text = type (text)
+    if t_text == "string" then
         words = lpegmatch(p_words, text)
         if #words < 2 then
             return { text }
         end
-    else
+    elseif t_text == "table" then
         words = text
-        if #words < 2 then
-            return words
+        if #text < 2 then
+            return text
         end
     end
 
@@ -512,42 +520,24 @@ local print_field = function (key, val)
     end
 end
 
-local display_names = function (names)
-    print_heading("Font Metadata", 2)
-    for i=1, #names do
-        local lang, namedata = names[i].lang, names[i].names
-        print_heading(stringformat("Language: %s ", i, lang), 3)
-        texiowrite_nl ""
-        if namedata then
-            for field, value in next, namedata do
-                print_field(field, value)
-            end
-        end
-    end
-end
-
 --- see luafflib.c
 local general_fields = {
     --- second: l -> literal | n -> length | d -> date
     { "fullname",            "l", "font name"           },
     { "version",             "l", "font version"        },
-    { "creationtime",        "d", "creation time"       },
-    { "modificationtime",    "d", "modification time"   },
-    { "subfonts",            "n", "number of subfonts"  },
-    { "glyphcnt",            "l", "number of glyphs"    },
+    { "width",               "l", "width"               },
+    { "averagewidth",        "l", "average width"       },
+    { "panosewidth",         "l", "panose width"        },
     { "weight",              "l", "weight indicator"    },
-    { "design_size",         "l", "design size"         },
-    { "design_range_bottom", "l", "design size min"     },
-    { "design_range_top",    "l", "design size max"     },
-    { "fontstyle_id",        "l", "font style id"       },
-    { "fontstyle_name",      "S", "font style name"     },
-    { "strokewidth",         "l", "stroke width"        },
-    { "units_per_em",        "l", "units per em"        },
-    { "ascent",              "l", "ascender height"     },
-    { "descent",             "l", "descender height"    },
-    { "comments",            "l", "comments"            },
-    { "os2_version",         "l", "os2 version"         },
-    { "sfd_version",         "l", "sfd version"         },
+    { "pfmweight",           "l", "numerical weight"    },
+    { "panoseweight",        "l", "panose weight"       },
+    { "designsize",          "l", "design size"         },
+    { "minsize",             "l", "design size min"     },
+    { "maxsize",             "l", "design size max"     },
+    { "units",               "l", "units per em"        },
+    { "ascender",            "l", "ascender height"     },
+    { "descender",           "l", "descender height"    },
+    { "capheight",           "l", "capital height"      },
 }
 
 local display_general = function (fullinfo)
@@ -607,7 +597,7 @@ local print_features = function (features)
         for script, languages in next, data do
             local field     = stringformat(key_fmt, script).. fieldseparator .. " "
             local wd_field  = #field
-            local lines     = reflow(languages.list, textwidth - wd_field)
+            local lines     = reflow(tablekeys(languages), textwidth - wd_field)
             local indent    = stringrep(" ", wd_field)
             texiowrite_nl(field)
             texiowrite(lines[1])
@@ -620,88 +610,40 @@ local print_features = function (features)
     end
 end
 
-local extract_feature_info = function (set)
-    local collected = { }
-    for i=1, #set do
-        local features = set[i].features
-        if features then
-            for j=1, #features do
-                local feature   = features[j]
-                local scripts   = feature.scripts
-                local tagname   = stringlower(feature.tag)
-                local entry     = collected[tagname] or { }
-
-                for k=1, #scripts do
-                    local script     = scripts[k]
-                    local scriptname = stringlower(script.script)
-                    local c_script   = entry[scriptname] or {
-                                            list = { },
-                                            set  = { },
-                                        }
-                    local list, set  = c_script.list, c_script.set
-
-                    for l=1, #script.langs do
-                        local langname = stringlower(script.langs[l])
-                        if not set[langname] then
-                            list[#list+1] = langname
-                            set[langname] = true
-                        end
-                    end
-                    entry[scriptname] = c_script
-                end
-                collected[tagname]  = entry
-            end
-        end
-    end
-    return collected
-end
-
 local display_feature_set = function (set)
-    local collected = extract_feature_info(set)
-    print_features(collected)
+    print_features(set)
 end
 
-local display_features = function (gsub, gpos)
+local display_features_type = function (id, feat)
+    if next (feat) then
+        print_heading(id, 3)
+        display_feature_set(feat)
+        return true
+    end
+    return false
+end
+
+local display_features = function (features)
     texiowrite_nl ""
+    print_heading("Features", 2)
 
-    if gsub or gpos then
-        print_heading("Features", 2)
-
-        if gsub then
-            print_heading("GSUB Features", 3)
-            display_feature_set(gsub)
-        end
-
-        if gpos then
-            print_heading("GPOS Features", 3)
-            display_feature_set(gpos)
-        end
+    if not display_features_type ("GSUB Features", features.gsub)
+    or not display_features_type ("GPOS Features", features.gpos)
+    then
+        texiowrite_nl("font defines neither gsub nor gpos features")
     end
 end
 
-local show_full_info = function (path, subfont, warnings)
-    local rawinfo, warn = fontloader.open(path, subfont)
-    if warnings then
-        show_fontloader_warnings(warn)
-    end
+local show_full_info = function (path, subfont)
+    local rawinfo, warn = fonts.handlers.otf.readers.loadfont (path, subfont)
     if not rawinfo then
         texiowrite_nl(stringformat([[cannot open font %s]], path))
         return
     end
-    local fontdata = { }
-    local fullinfo = fontloader.to_table(rawinfo)
-    local fields = fontloader.fields(rawinfo)
-    fontloader.close(rawinfo)
-    display_names(fullinfo.names)
-    display_general(fullinfo)
-    display_features(fullinfo.gsub, fullinfo.gpos)
+    display_general(rawinfo.metadata)
+    display_features(rawinfo.resources.features)
 end
 
---- Subfonts returned by fontloader.info() do not correspond
---- to the actual indices required by fontloader.open(), so
---- we try and locate the correct one by matching the request
---- against the full name.
-
 local subfont_by_name
 subfont_by_name = function (lst, askedname, n)
     if not n then
@@ -713,7 +655,7 @@ subfont_by_name = function (lst, askedname, n)
         if fonts.names.sanitize_fontname (font.fullname) == askedname then
             return font
         end
-        return subfont_by_name (lst, askedname, n+1)
+        return subfont_by_name (lst, askedname, n + 1)
     end
     return false
 end
@@ -721,12 +663,12 @@ end
 --[[doc--
 The font info knows two levels of detail:
 
-    a)  basic information returned by fontloader.info(); and
+    a)  basic information returned by readers.getinfo(); and
     b)  detailed information that is a subset of the font table
-        returned by fontloader.open().
+        returned by readers.loadfont().
 --doc]]--
 
-local show_font_info = function (basename, askedname, detail, warnings)
+local show_font_info = function (basename, askedname, detail, subfont)
     local filenames = fonts.names.data().files
     local index     = filenames.base[basename]
     local fullname  = filenames.full[index]
@@ -735,7 +677,11 @@ local show_font_info = function (basename, askedname, detail, warnings)
         fullname = resolvers.findfile(basename)
     end
     if fullname then
-        local shortinfo = fontloader.info(fullname)
+        local shortinfo = fonts.handlers.otf.readers.getinfo (fullname, {
+                            subfont        = subfont,
+                            platformnames  = true,
+                            rawfamilynames = true,
+                        })
         local nfonts   = #shortinfo
         if nfonts > 0 then -- true type collection
             local subfont
@@ -748,24 +694,25 @@ local show_font_info = function (basename, askedname, detail, warnings)
             if subfont then
                 show_info_items(subfont)
                 if detail == true then
-                    show_full_info(fullname, subfont, warnings)
+                    show_full_info(fullname, subfont)
                 end
             else -- list all subfonts
                 logreport (true, 1, "resolve",
                            [[%s is a font collection]], basename)
                 for subfont = 1, nfonts do
                     logreport (true, 1, "resolve",
-                               [[Showing info for font no. %d]], n)
+                               [[Showing info for font no. %d]],
+                               subfont)
                     show_info_items(shortinfo[subfont])
                     if detail == true then
-                        show_full_info(fullname, subfont, warnings)
+                        show_full_info(fullname, subfont)
                     end
                 end
             end
         else
             show_info_items(shortinfo)
             if detail == true then
-                show_full_info(fullname, subfont, warnings)
+                show_full_info(fullname, subfont)
             end
         end
     else
@@ -843,7 +790,7 @@ actions.blacklist = function (job)
 end
 
 actions.generate = function (job)
-    local _ = fonts.names.update (fontnames, job.force_reload, job.dry_run)
+    local _ = fonts.names.update ({ }, job.force_reload, job.dry_run)
     local namedata = fonts.names.data ()
     if namedata then
         logreport ("info", 2, "db",
@@ -1216,16 +1163,22 @@ actions.query = function (job)
         tmpspec.size = 655360 --- assume 10pt
     end
 
-    local foundname, subfont, success
+    local foundname, subfont, success, needle
 
-    if tmpspec.lookup == "name"
-    or tmpspec.lookup == "anon" --- not *exactly* as resolvers.anon
-    then
-        foundname, _, success = fonts.names.lookup_font_name (tmpspec)
-        if foundname then
-            foundname, _, success = fonts.names.lookup_font_file (foundname)
+    if tmpspec.lookup == "name" then
+        if fonts.definers.resolvers.name (tmpspec) then
+            needle = tmpspec.resolved
+        end
+    elseif tmpspec.lookup == "anon" then
+        if fonts.definers.resolvers.anon (tmpspec) then
+            needle = tmpspec.resolved or tmpspec.name
         end
     elseif tmpspec.lookup == "file" then
+        needle  = tmpspec.name
+        subfont = tmpspec.sub
+    end
+
+    if needle then
         foundname, _, success = fonts.names.lookup_font_file (tmpspec.name)
     end
 
@@ -1240,7 +1193,8 @@ actions.query = function (job)
                        "Resolved file name %q", foundname)
         end
         if job.show_info then
-            show_font_info (foundname, query, job.full_info, job.warnings)
+            logreport (false, 3, "resolve", "Dump extra info.")
+            show_font_info (foundname, query, job.full_info, subfont)
             iowrite "\n"
         end
     else
@@ -1465,7 +1419,6 @@ local process_cmdline = function ( ) -- unit -> jobspec
     local result = { -- jobspec
         force_reload = nil,
         full_info    = false,
-        warnings     = false,
         criterion    = "",
         query        = "",
         log_level    = nil,
@@ -1506,10 +1459,9 @@ local process_cmdline = function ( ) -- unit -> jobspec
         update             = "u",
         verbose            = 1,
         version            = "V",
-        warnings           = "w",
     }
 
-    local short_options = "bcDfFiIlLnpqRSuvVhw"
+    local short_options = "bcDfFiIlLnpqRSuvVh"
 
     local options, _, optarg =
         alt_getopt.get_ordered_opts (arg, short_options, long_options)
@@ -1541,12 +1493,7 @@ local process_cmdline = function ( ) -- unit -> jobspec
             if lvl then
                 lvl = tonumber(lvl)
                 result.log_level = lvl
-                if lvl > 2 then
-                    result.warnings = true
-                end
             end
-        elseif v == "w" then
-            result.warnings = true
         elseif v == "log" then
             local str = optarg[n]
             if str then
