@@ -1,9 +1,8 @@
 #!/bin/sh
-# $Id: fmtutil-sys.sh 34641 2014-07-17 16:38:18Z karl $
-# fmtutil-sys: Thomas Esser, public domain.
-
-# wrapper script for fmtutil with TEXMFVAR and TEXMFCONFIG set to
-#   TEXMFSYSVAR / TEXMFSYSCONFIG
+# $Id: fmtutil-sys.sh 36962 2015-04-20 01:48:35Z preining $
+# fmtutil-sys - arrange for fmtutil to affect system directories.
+# Maintained in Master/texmf-dist/scripts/texlive/
+# Public domain.  Originally written by Thomas Esser.
 
 test -f /bin/ksh && test -z "$RUNNING_KSH" \
   && { UNAMES=`uname -s`; test "x$UNAMES" = xULTRIX; } 2>/dev/null \
@@ -15,20 +14,12 @@ test -f /bin/bsh && test -z "$RUNNING_BSH" \
   && { RUNNING_BSH=true; export RUNNING_BSH; exec /bin/bsh $0 ${1+"$@"}; }
 unset RUNNING_BSH
 
-# hack around a bug in zsh:
-test -n "${ZSH_VERSION+set}" && alias -g '${1+"$@"}'='"$@"'
-export PATH
-
 # preferentially use subprograms from our own directory.
 mydir=`echo "$0" | sed 's,/[^/]*$,,'`
 mydir=`cd "$mydir" && pwd`
 PATH="$mydir:$PATH"; export PATH
 
-v=`kpsewhich -var-value TEXMFSYSVAR`
-c=`kpsewhich -var-value TEXMFSYSCONFIG`
+# hack around a bug in zsh:
+test -n "${ZSH_VERSION+set}" && alias -g '${1+"$@"}'='"$@"'
 
-TEXMFVAR="$v"
-TEXMFCONFIG="$c"
-export TEXMFVAR TEXMFCONFIG
-
-exec fmtutil ${1+"$@"}
+exec fmtutil --sys ${1+"$@"}
