@@ -36,7 +36,7 @@ local registertracker = trackers.register
 local trace_injections  = false  registertracker("fonts.injections",         function(v) trace_injections = v end)
 local trace_marks       = false  registertracker("fonts.injections.marks",   function(v) trace_marks      = v end)
 local trace_cursive     = false  registertracker("fonts.injections.cursive", function(v) trace_cursive    = v end)
-local trace_spaces      = false  registertracker("otf.spaces",               function(v) trace_spaces  = v end)
+local trace_spaces      = false  registertracker("fonts.injections.spaces",  function(v) trace_spaces  = v end)
 
 -- use_advance is just an experiment: it makes copying glyphs (instead of new_glyph) dangerous
 
@@ -377,7 +377,8 @@ function injections.setkern(current,factor,rlmode,x,injection)
     end
 end
 
-function injections.setmark(start,base,factor,rlmode,ba,ma,tfmbase,mkmk) -- ba=baseanchor, ma=markanchor
+function injections.setmark(start,base,factor,rlmode,ba,ma,tfmbase,mkmk,checkmark) -- ba=baseanchor, ma=markanchor
+
     local dx, dy = factor*(ba[1]-ma[1]), factor*(ba[2]-ma[2])
     nofregisteredmarks = nofregisteredmarks + 1
     if rlmode >= 0 then
@@ -398,6 +399,7 @@ function injections.setmark(start,base,factor,rlmode,ba,ma,tfmbase,mkmk) -- ba=b
                 i.markbase     = nofregisteredmarks
                 i.markbasenode = base
                 i.markmark     = mkmk
+                i.checkmark    = checkmark
             end
         else
             p.injections = {
@@ -407,6 +409,7 @@ function injections.setmark(start,base,factor,rlmode,ba,ma,tfmbase,mkmk) -- ba=b
                 markbase     = nofregisteredmarks,
                 markbasenode = base,
                 markmark     = mkmk,
+                checkmark    = checkmark,
             }
         end
     else
@@ -418,6 +421,7 @@ function injections.setmark(start,base,factor,rlmode,ba,ma,tfmbase,mkmk) -- ba=b
                 markbase     = nofregisteredmarks,
                 markbasenode = base,
                 markmark     = mkmk,
+                checkmark    = checkmark,
             },
         }
     end
@@ -1062,11 +1066,22 @@ local function inject_everything(head,where)
                 ox = px - pn.markx
              -- report_injections("l2r case 3: %p",ox)
          -- end
-            local wn = getfield(n,"width") -- in arial marks have widths
-            if wn ~= 0 then
-                -- bad: we should center
-                pn.leftkern  = -wn/2
-                pn.rightkern = -wn/2
+            if pn.checkmark then
+                local wn = getfield(n,"width") -- in arial marks have widths
+                if wn ~= 0 then
+                    wn = wn/2
+                    if trace_injections then
+                        report_injections("correcting non zero width mark %C",getchar(n))
+                    end
+                    -- -- bad: we should center
+                    -- pn.leftkern  = -wn
+                    -- pn.rightkern = -wn
+                    -- -- we're too late anyway as kerns are already injected so
+                    -- -- we do it the ugly way (no checking if the previous is
+                    -- -- already a kern) .. maybe we should fix the font instead
+                    insert_node_before(n,n,newkern(-wn))
+                    insert_node_after(n,n,newkern(-wn))
+                end
             end
         end
         local oy = getfield(n,"yoffset") + getfield(p,"yoffset") + pn.marky
@@ -1092,6 +1107,10 @@ local function inject_everything(head,where)
                             nofmarks = nofmarks + 1
                             marks[nofmarks] = current
                         else
+                            local yoffset = i.yoffset
+                            if yoffset and yoffset ~= 0 then
+                                setfield(current,"yoffset",yoffset)
+                            end
                             if hascursives then
                                 local cursivex = i.cursivex
                                 if cursivex then
@@ -1144,10 +1163,6 @@ local function inject_everything(head,where)
                                 end
                             end
                             -- left|glyph|right
-                            local yoffset = i.yoffset
-                            if yoffset and yoffset ~= 0 then
-                                setfield(current,"yoffset",yoffset)
-                            end
                             local leftkern = i.leftkern
                             if leftkern and leftkern ~= 0 then
                                 insert_node_before(head,current,newkern(leftkern))
@@ -1162,7 +1177,7 @@ local function inject_everything(head,where)
                         local i = p.emptyinjections
                         if i then
                             -- glyph|disc|glyph (special case)
--- okay?
+                            -- okay?
                             local rightkern = i.rightkern
                             if rightkern and rightkern ~= 0 then
                                 if next and getid(next) == disc_code then
@@ -1422,6 +1437,48 @@ function nodes.injections.setspacekerns(font,sequence)
     end
 end
 
+local getthreshold
+
+if context then
+
+    local threshold  =  1 -- todo: add a few methods for context
+    local parameters = fonts.hashes.parameters
+
+    directives.register("otf.threshold", function(v) threshold = tonumber(v) or 1 end)
+
+    getthreshold  = function(font)
+        local p = parameters[font]
+        local f = p.factor
+        local s = p.spacing
+        local t = threshold * (s and s.width or p.space or 0) - 2
+        return t > 0 and t or 0, f
+    end
+
+else
+
+    injections.threshold = 0
+
+    getthreshold  = function(font)
+        local p = fontdata[font].parameters
+        local f = p.factor
+        local s = p.spacing
+        local t = injections.threshold * (s and s.width or p.space or 0) - 2
+        return t > 0 and t or 0, f
+    end
+
+end
+
+injections.getthreshold = getthreshold
+
+function injections.isspace(n,threshold)
+    if getid(n) == glue_code then
+        local w = getfield(n,"width")
+        if threshold and w > threshold then -- was >=
+            return 32
+        end
+    end
+end
+
 local function injectspaces(head)
 
     if not triggers then
@@ -1438,18 +1495,11 @@ local function injectspaces(head)
     local rightkern  = false
 
     local function updatefont(font,trig)
-     -- local resources  = resources[font]
-     -- local spacekerns = resources.spacekerns
-     -- if spacekerns then
-     --     leftkerns  = spacekerns.left
-     --     rightkerns = spacekerns.right
-     -- end
         leftkerns  = trig.left
         rightkerns = trig.right
-        local par  = fontdata[font].parameters -- fallback for generic
-        factor     = par.factor
-        threshold  = par.spacing.width - 1 -- get rid of rounding errors
         lastfont   = font
+        threshold,
+        factor     = getthreshold(font)
     end
 
     for n in traverse_id(glue_code,tonut(head)) do
@@ -1469,7 +1519,7 @@ local function injectspaces(head)
             end
         end
         if prevchar then
-            local font = getfont(next)
+            local font = getfont(prev)
             local trig = triggers[font]
             if trig then
                 if lastfont ~= font then
@@ -1482,7 +1532,7 @@ local function injectspaces(head)
         end
         if leftkern then
             local old = getfield(n,"width")
-            if old >= threshold then
+            if old > threshold then
                 if rightkern then
                     local new = old + (leftkern + rightkern) * factor
                     if trace_spaces then
@@ -1501,7 +1551,7 @@ local function injectspaces(head)
             leftkern  = false
         elseif rightkern then
             local old = getfield(n,"width")
-            if old >= threshold then
+            if old > threshold then
                 local new = old + rightkern * factor
                 if trace_spaces then
                     report_spaces("[%p -> %p] %C",nextchar,old,new)
@@ -1522,6 +1572,7 @@ function injections.handler(head,where)
     if triggers then
         head = injectspaces(head)
     end
+    -- todo: marks only run too
     if nofregisteredmarks > 0 or nofregisteredcursives > 0 then
         if trace_injections then
             report_injections("injection variant %a","everything")
